'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = readToken;

var _acorn = require('acorn');

function readToken(nextMethod) {
  return function ha(code) {
    // https://github.com/ternjs/acorn/blob/master/src/tokenize.js#L313-L323
    switch (code) {
      case 65288:
        ++this.pos;return this.finishToken(_acorn.tokTypes.parenL);
      case 65289:
        ++this.pos;return this.finishToken(_acorn.tokTypes.parenR);
      case 65307:
        ++this.pos;return this.finishToken(_acorn.tokTypes.semi);
      case 65292:
        ++this.pos;return this.finishToken(_acorn.tokTypes.comma);
      case 12308:case 12304:case 12310:
        // 〔 【 〖
        ++this.pos;return this.finishToken(_acorn.tokTypes.bracketL);
      case 12309:case 12305:case 12311:
        // 〕 】 〗
        ++this.pos;return this.finishToken(_acorn.tokTypes.bracketR);
      case 65371:
        ++this.pos;return this.finishToken(_acorn.tokTypes.braceL);
      case 65373:
        ++this.pos;return this.finishToken(_acorn.tokTypes.braceR);
      case 65306:
        ++this.pos;return this.finishToken(_acorn.tokTypes.colon);
      case 65311:
        ++this.pos;return this.finishToken(_acorn.tokTypes.question);

      // https://github.com/ternjs/acorn/blob/master/src/tokenize.js#L343-L344
      // To make this work, we need to modify readString a little bit. See below.
      case 12300:
        return this.readString(12301); // Start with 「, so end with 」.
      case 12302:
        return this.readString(12303); // Start with 『, so end with 』.
      case 8216:
        return this.readString(8217); // Start with ‘, so end with ’.
      case 8220:
        return this.readString(8221); // Start with “, so end with ”.
      default:
    }
    return nextMethod.call(this, code);
  };
} /* eslint-disable no-plusplus */

module.exports = exports['default'];